'''
 Tool for BeamNG moders which replaces a substring
 in file names, folder names and in the content of beamng text files.
 from a given root folder or file.

 Only the content of files of the following types will be edited:
 json, jbeam, prefab, txt, cs, lua, dml, pc, dae
 
 Don't worry about case:
 before becomes after,
 Before becomes After,
 BEFORE becomes AFTER,
 no matter how you wrote bEfOrE and afTEr.

 If you ever get a Win Error 5 access is denied:
 It's a known problem between Python's file renaming function and Windows
 permissions. If it happens, try stopping backups, scans or anything that
 might be reading your mod files. At the very worst, try rebooting your
 computer. Running the script with administrative rights can also help.

 Happy moding!

 __  ___  ___  ______ _____   ___  ______ _____ _   _  _____ 
|  \/  | / _ \ | ___ \  __ \ / _ \ | ___ \_   _| \ | ||  ___|
| .  . |/ /_\ \| |_/ / |  \// /_\ \| |_/ / | | |  \| || |__  
| |\/| ||  _  ||    /| | __ |  _  ||    /  | | | . ` ||  __| 
| |  | || | | || |\ \| |_\ \| | | || |\ \ _| |_| |\  || |___ 
\_|  |_/\_| |_/\_| \_|\____/\_| |_/\_| \_|\___/\_| \_/\____/

'''
import os, sys
import tkinter as tk


class Replacer:
    def __init__(self, master):
        self.master = master
        self.root_dir = ""
        self.old_name = ""
        self.new_name = ""
        self.create_interface()
        self.current_modifications = []

    def create_interface(self):
        self.master.title("Mod renamer")
        self.el = {}
        tk.Label(self.master, text="Path: ").grid(row=0)
        self.el['sv_dir'] = tk.StringVar()
        self.el['sv_dir'].set(self.root_dir)
        self.el['e_dir'] = tk.Entry(self.master, textvariable=self.el['sv_dir']).grid(row=0, column=1)
        tk.Label(self.master, text="Old name: ").grid(row=1)
        self.el['sv_old_name'] = tk.StringVar()
        self.el['sv_old_name'].set(self.old_name)
        self.el['e_old_name'] = tk.Entry(self.master, textvariable=self.el['sv_old_name']).grid(row=1, column=1)
        tk.Label(self.master, text="New name: ").grid(row=2)
        self.el['sv_new_name'] = tk.StringVar()
        self.el['sv_new_name'].set(self.new_name)
        self.el['e_new_name'] = tk.Entry(self.master, textvariable=self.el['sv_new_name']).grid(row=2, column=1)
        self.el['ok_button'] = tk.Button(self.master, text="Ok", command=self.execute).grid(row=3, column=1)
        self.master.bind('<Return>', self.enter_key)



    def enter_key(self, *_):
        self.execute()
        
    def execute(self):
        self.root_dir = self.el['sv_dir'].get()
        self.old_name = self.el['sv_old_name'].get()
        self.new_name = self.el['sv_new_name'].get()
        self.root_dir = Replacer.remove_quotes(self.root_dir)
        if self.empty_entry():
            print("Fill all the fields.")
        elif not os.path.exists(self.root_dir):
            print("Root folder doesn't exist.")
        else:
            self.root_dir = self.rename(self.root_dir)
            if not os.path.isdir(self.root_dir):
                self.replace_in_file(self.root_dir)
            else:
                self.walk(self.root_dir)
            self.print_modifications()
            
    def walk(self, root):
        for el in os.listdir(root):
            path = os.path.join(root, el)
            if os.path.isdir(path):
                path = self.rename(path)
                self.walk(path)
            else:
                path = self.rename(path)
                if Replacer.is_beamng_text_file(path):
                    self.replace_in_file(path)
                elif path.endswith(".cdae"):
                    os.remove(path)

    def rename(self, file):
        dirname = os.path.dirname(file)
        basename = os.path.basename(file)
        new_basename = self.replace(basename)
        new_file = os.path.join(dirname, new_basename)
        if not file == new_file:
            os.rename(file, new_file)
            self.current_modifications.append("Renamed: {} --> {}".format(os.path.basename(file), os.path.basename(new_file)))
        return new_file

    def replace_in_file(self, path):
        f = open(path, "r")
        text = f.read()
        f.close()
        new_text = self.replace(text)
        if not new_text == text:
            self.current_modifications.append("Modified: {}".format(os.path.basename(path)))
        f = open(path, "w")
        f.write(new_text)
        f.close()

    def print_modifications(self):
        if len(self.current_modifications) > 0:
            for m in self.current_modifications:
                print(m)
            print("Done.\n\n")
            self.current_modifications = []
        else:
            print("No match found.\n\n")

    def replace(self, text):
        text = text.replace(self.old_name.upper(), self.new_name.upper())
        text = text.replace(self.old_name.lower(), self.new_name.lower())
        text = text.replace(Replacer.capitalize_initial(self.old_name), Replacer.capitalize_initial(self.new_name))
        return text

    def empty_entry(self):
        return len(self.root_dir) == 0 or len(self.old_name) == 0 or len(self.new_name) == 0

    @staticmethod
    def capitalize_initial(text):
        return text[:1].upper() + text[1:].lower()

    @staticmethod
    def remove_quotes(path):
        if path[:1] == '"' and path[-1:] == '"':
            path = path[1:-1]
        return path
    
    @staticmethod
    def is_beamng_text_file(path):
        return path.endswith(".json") or path.endswith(".jbeam") or path.endswith(".prefab") or \
                path.endswith(".txt") or path.endswith(".cs") or path.endswith(".lua") or \
                path.endswith(".dml") or path.endswith(".pc") or path.endswith(".dae")
   
if __name__ == '__main__':
    root = tk.Tk()
    r = Replacer(root)
    root.mainloop()
